@@ -3,9 +3,10 @@ from functools import partial
 
 import django
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Manager, ForeignKey, OneToOneField
+from django.db.models import Manager, ForeignKey, OneToOneField, options
 from django.db.models.base import ModelBase
 from django.db.models.signals import post_init
+from django.utils.functional import cached_property
 from six import with_metaclass
 
 from modeltranslation import settings as mt_settings
@@ -124,6 +125,15 @@ class TranslationOptions(with_metaclass(FieldsAggregationMetaClass, object)):
         return '%s: %s + %s' % (self.__class__.__name__, local, inherited)
 
 
+class MultilingualOptions(options.Options):
+
+    @cached_property
+    def base_manager(self):
+        manager = super(MultilingualOptions, self).base_manager
+        patch_manager_class(manager)
+        return manager
+
+
 def add_translation_fields(model, opts):
     """
     Monkey patches the original model class to provide additional fields for
@@ -190,6 +200,18 @@ def patch_manager_class(manager):
                     self._constructor_args[1],  # kwargs
                 )
 
+            def __hash__(self):
+                return id(self)
+
+            def __eq__(self, other):
+                if isinstance(other, NewMultilingualManager):
+                    return self._old_module == other._old_module and \
+                        self._old_class == other._old_class
+                if hasattr(other, "__module__") and hasattr(other, "__class__"):
+                    return self._old_module == other.__module__ and \
+                        self._old_class == other.__class__.__name__
+                return False
+
         manager.__class__ = NewMultilingualManager
 
 
@@ -221,8 +243,8 @@ def add_manager(model):
             # model._default_manager is not the same instance as one of managers, but it
             # share the same class.
             model._default_manager.__class__ = current_manager.__class__
-    patch_manager_class(model._base_manager)
-    model._meta.base_manager_name = 'objects'
+
+    model._meta.__class__ = MultilingualOptions
     model._meta._expire_cache()
 
 
@@ -555,6 +577,7 @@ class Translator(object):
         """
         if model not in self._registry:
             # Create a new type for backwards compatibility.
+
             opts = type("%sTranslationOptions" % model.__name__,
                         (opts_class or TranslationOptions,), options)(model)
 
